@@ -17,8 +17,8 @@
 #
 #***********************************************************************
 
-use warnings;
 use strict;
+use warnings;
 
 # Move site library directory ahead of default library directory in @INC.
 # That's so we can sanely package our own version of MIME::Base64 that
@@ -322,10 +322,11 @@ sub main {
     }
 
     $ValidateIPHeader = "";
-    if (open(IN, '<', '/etc/mail/mimedefang-ip-key')) {
-	$ValidateIPHeader = <IN>;
+    my $in;
+    if (open($in, '<', '/etc/mail/mimedefang-ip-key')) {
+	$ValidateIPHeader = <$in>;
 	chomp($ValidateIPHeader);
-	close(IN);
+	close($in);
     }
 
     # These are set unconditionally; filter() can change them.
@@ -839,7 +840,7 @@ sub do_scan {
     $parser->filer($filer);
 
     # Parse the input stream:
-    if (!open(FILE, '<', $file)) {
+    if (not -r $file) {
 	fatal("couldn't open $file: $!");
 	signal_complete();
 	return -1;
@@ -849,9 +850,8 @@ sub do_scan {
 	$parser->max_parts($MaxMIMEParts);
     }
     push_status_tag("Parsing Message");
-    $entity = $parser->parse(\*FILE);
+    $entity = $parser->parse_open($file);
     pop_status_tag();
-    close FILE;
 
     if (!defined($entity) && $MaxMIMEParts > 0) {
 	# Message is too complex; bounce it
@@ -986,7 +986,8 @@ sub do_scan {
     }
 
     if ($Changed || $Rebuild) {
-	if (!open(OUT, ">", "NEWBODY")) {
+	my $fh = IO::File->new("NEWBODY", '>:');
+	if (not $fh) {
 	    fatal("Can't open NEWBODY: $!");
 	    signal_complete();
 	    return -1;
@@ -1002,10 +1003,9 @@ sub do_scan {
 		$rebuilt->make_singlepart();
 	}
 	push_status_tag("Writing new body");
-	$rebuilt->print_body(\*OUT);
+	$rebuilt->print_body($fh);
 	pop_status_tag();
-	close(OUT);
-
+	$fh->close;
 
 	# Write new content-type header in case we've changed the type.
 	my $ct = $rebuilt->head->get('Content-Type');
