@@ -32,6 +32,7 @@ You're limited only by your imagination. If you can think of it and code it in P
 
 package Mail::MIMEDefang;
 
+## no critic (TestingAndDebugging::RequireUseStrict)
 use warnings;
 no warnings qw(once uninitialized);
 
@@ -40,13 +41,16 @@ require Exporter;
 use Carp;
 use Errno qw(ENOENT EACCES);
 use File::Spec;
+use IO::File;
+use MIME::Entity;
 use MIME::WordDecoder;
+use Socket;
 
 our @ISA = qw(Exporter);
 our @EXPORT;
 our @EXPORT_OK;
 
-our $VERSION = '3.4';
+our $VERSION = '3.6';
 
 @EXPORT = qw{
       $AddWarningsInline @StatusTags
@@ -86,8 +90,8 @@ our $VERSION = '3.4';
       signal_changed signal_unchanged md_syslog md_graphdefang_log
       write_result_line in_message_context in_filter_context in_filter_wrapup
       in_filter_end percent_decode percent_encode percent_encode_for_graphdefang
-      send_mail send_quarantine_notifications signal_complete send_admin_mail
-      md_version set_status_tag
+      send_mail send_multipart_mail send_quarantine_notifications signal_complete send_admin_mail
+      md_version set_status_tag read_commands_file
     };
 
 @EXPORT_OK = qw{
@@ -208,14 +212,21 @@ Prints a message to syslog(3) using the specified facility
 	{
 		my ($tag, $facility) = @_;
 
+		local $@;
 		if( ! defined $_openlogsub ) {
 			# Try Unix::Syslog first, then Sys::Syslog
-			eval qq{use Unix::Syslog qw( :macros ); };
+			eval {
+			  require Unix::Syslog;
+			};
 			if(!$@) {
+				Unix::Syslog->import(qw( :macros ));
 				($_openlogsub, $_syslogsub) = _wrap_for_unix_syslog();
 			} else {
-				eval qq{use Sys::Syslog ();};
+				eval {
+				  require Sys::Syslog;
+				};
 				if(!$@) {
+					Sys::Syslog->import();
 					($_openlogsub, $_syslogsub) = _wrap_for_sys_syslog();
 				} else {
 					croak q{Unable to detect either Unix::Syslog or Sys::Syslog};
@@ -339,11 +350,12 @@ Prints a message to syslog(3) using the specified facility
         sub _fac_to_num
 	{
 		my ($thing) = @_;
+		local $@;
 		return if exists $blacklisted{$thing};
 		$thing = $special{$thing} if exists $special{$thing};
 		$thing = 'LOG_' . uc($thing);
 		return unless grep { $_ eq $thing } @ {$Unix::Syslog::EXPORT_TAGS{macros} };
-		return eval "Unix::Syslog::$thing()";
+		return eval "Unix::Syslog::$thing()"; ## no critic
 	}
 }
 
@@ -407,26 +419,29 @@ sub md_graphdefang_log
     my $id = percent_encode_for_graphdefang($MsgID);
     my $subj;
     if($utf8_decode eq 1) {
-      $subj = mime_to_perl_string($Subject);
-      $event = mime_to_perl_string($event);
-      $value1 = mime_to_perl_string($value1);
-      $value2 = mime_to_perl_string($value2);
-      $subj =~ s/\P{Print}//g;
-      $event =~ s/\P{Print}//g;
-      $value1 =~ s/\P{Print}//g;
-      $value2 =~ s/\P{Print}//g;
-      if (utf8::is_utf8($subj)) {
-        utf8::encode($subj);
-      }
-      if (utf8::is_utf8($event)) {
-        utf8::encode($event);
-      }
-      if (utf8::is_utf8($value1)) {
-        utf8::encode($value1);
-      }
-      if (utf8::is_utf8($value2)) {
-        utf8::encode($value2);
-      }
+      eval {
+        no warnings 'utf8';
+        $subj = mime_to_perl_string($Subject);
+        $event = mime_to_perl_string($event);
+        $value1 = mime_to_perl_string($value1);
+        $value2 = mime_to_perl_string($value2);
+        $subj =~ s/\P{Print}//g;
+        $event =~ s/\P{Print}//g;
+        $value1 =~ s/\P{Print}//g;
+        $value2 =~ s/\P{Print}//g;
+        if (utf8::is_utf8($subj)) {
+          utf8::encode($subj);
+        }
+        if (utf8::is_utf8($event)) {
+          utf8::encode($event);
+        }
+        if (utf8::is_utf8($value1)) {
+          utf8::encode($value1);
+        }
+        if (utf8::is_utf8($value2)) {
+          utf8::encode($value2);
+        }
+      };
     } else {
       $subj = percent_encode_for_graphdefang($Subject);
       $event = percent_encode_for_graphdefang($event);
@@ -464,30 +479,59 @@ like SpamAssassin, rbl checks, zip file listing and HTML parsing.
 
 sub detect_and_load_perl_modules() {
     my ($use_sa, $use_html, $use_zip, $use_dns);
+    local $@;
     if (!defined($Features{"AutoDetectPerlModules"}) or
       $Features{"AutoDetectPerlModules"}) {
       if (!defined($Features{"SpamAssassin"}) or ($Features{"SpamAssassin"} eq 1)) {
-        (eval 'use Mail::SpamAssassin (); $use_sa = 1;')
-        or $use_sa = 0;
+        eval {
+	  require Mail::SpamAssassin;
+        };
+        if($@) {
+	  $use_sa = 0;
+	} else {
+	  Mail::SpamAssassin->import();
+	  $use_sa = 1;
+	}
       }
       $Features{"SpamAssassin"} = $use_sa;
       if (!defined($Features{"HTML::Parser"}) or ($Features{"HTML::Parser"} eq 1)) {
-        (eval 'use HTML::Parser; $use_html = 1;')
-        or $use_html = 0;
+        eval {
+	  require HTML::Parser;
+	};
+	if($@) {
+	  $use_html = 0;
+	} else {
+	  HTML::Parser->import();
+	  $use_html = 1;
+	}
       }
       $Features{"HTML::Parser"} = $use_html;
       if (!defined($Features{"Archive::Zip"}) or ($Features{"Archive::Zip"} eq 1)) {
-        (eval 'use Archive::Zip; $use_zip = 1;')
-        or $use_zip = 0;
+        eval {
+	  require Archive::Zip;
+        };
+	if($@) {
+	  $use_zip = 0;
+	} else {
+	  Archive::Zip->import();
+	  $use_zip = 1;
+	}
       }
       $Features{"Archive::Zip"} = $use_zip;
       if (!defined($Features{"Net::DNS"}) or ($Features{"Net::DNS"} eq 1)) {
-        (eval 'use Net::DNS; $use_dns = 1;')
-        or $use_dns = 0;
+        eval {
+	  require Net::DNS;
+	};
+	if($@) {
+	  $use_dns = 0;
+	} else {
+	  Net::DNS->import();
+	  $use_dns = 1;
+        }
       }
       $Features{"Net::DNS"} = $use_dns;
       if(exists &Mail::MIMEDefang::Actions::md_init) {
-        Mail::MIMEDefang::Action::md_init();
+        Mail::MIMEDefang::Actions::md_init();
       }
       if(exists &Mail::MIMEDefang::Antispam::md_init) {
         Mail::MIMEDefang::Antispam::md_init();
@@ -543,7 +587,7 @@ sub init_status_tag
 {
 	return unless $DoStatusTags;
 
-	if(open(STATUS_HANDLE, ">&=3")) {
+	if(open(STATUS_HANDLE, ">&=", 3)) { ## no critic
 		STATUS_HANDLE->autoflush(1);
 	} else {
 		$DoStatusTags = 0;
@@ -893,13 +937,6 @@ sub send_quarantine_notifications {
   if ($QuarantineCount > 0 || $EntireMessageQuarantined) {
 	  my($body);
 	  my $hostname = Mail::MIMEDefang::Net::get_host_name();
-	  $body = "From: $DaemonName <$DaemonAddress>\n";
-	  $body .= "To: \"$AdminName\" <$AdminAddress>\n";
-	  $body .= Mail::MIMEDefang::RFC2822::gen_date_msgid_headers($QueueID, $hostname);
-	  $body .= "Auto-Submitted: auto-generated\n";
-	  $body .= "MIME-Version: 1.0\nContent-Type: text/plain\n";
-	  $body .= "Precedence: bulk\n";
-	  $body .= "Subject: $QuarantineSubject\n\n";
 	  if ($QuarantineCount >= 1) {
 	    $body .= "An e-mail had $QuarantineCount part";
 	    $body .= "s" if ($QuarantineCount != 1);
@@ -920,44 +957,54 @@ sub send_quarantine_notifications {
 	    $body .= "Recipient: $recip\n";
 	  }
  	  my $donemsg = 0;
+	  my $in;
 	  my $i;
 	  for ($i=0; $i<=$QuarantineCount; $i++) {
-	    if (open(IN, "<", "$QuarantineSubdir/MSG.$i")) {
+	    if (open($in, "<", "$QuarantineSubdir/MSG.$i")) {
 		    if (!$donemsg) {
 		      $body .= "Quarantine Messages:\n";
 		      $donemsg = 1;
 	 	    }
-		    while(<IN>) {
+		    while(<$in>) {
 		      $body .= $_;
 		    }
-		    close(IN);
+		    close($in);
 	    }
 	  }
 	  if ($donemsg) {
 	    $body .= "\n";
 	  }
 
-	  if (open(IN, "<", "$QuarantineSubdir/HEADERS")) {
+	  if (open($in, "<", "$QuarantineSubdir/HEADERS")) {
 	    $body .= "\n----------\nHere are the message headers:\n";
-	    while(<IN>) {
+	    while(<$in>) {
 		    $body .= $_;
 	    }
-	    close(IN);
+	    close($in);
 	  }
 	  for ($i=1; $i<=$QuarantineCount; $i++) {
-	    if (open(IN, "<", "$QuarantineSubdir/PART.$i.HEADERS")) {
+	    if (open($in, "<", "$QuarantineSubdir/PART.$i.HEADERS")) {
 		    $body .= "\n----------\nHere are the headers for quarantined part $i:\n";
-		    while(<IN>) {
+		    while(<$in>) {
 		      $body .= $_;
 		    }
-		    close(IN);
+		    close($in);
 	    }
 	  }
 	  if ($#Warnings >= 0) {
 	    $body .= "\n----------\nHere are the warning details:\n\n";
 	    $body .= "@Warnings";
 	  }
-	  send_mail($DaemonAddress, $DaemonName, $AdminAddress, $body);
+          my $mime = MIME::Entity->build(
+            From => "\"$DaemonName\" <$DaemonAddress>",
+            To => "\"$AdminName\" <$AdminAddress>",
+            Subject => $QuarantineSubject,
+            Encoding => 'quoted-printable',
+            Data => [$body],
+            Type => "text/plain");
+          $mime->head->add('Auto-Submitted', 'auto-generated');
+          $mime->head->add('Precedence', 'bulk');
+          send_mail($DaemonAddress, $DaemonName, $AdminAddress, $mime->stringify);
   }
 }
 
@@ -982,42 +1029,45 @@ Also mails any quarantine notifications and sender notifications.
 sub signal_complete {
   # Send notification to sender, if required
   if ($Sender ne '<>' && -r "NOTIFICATION") {
-	  my($body);
-	  my $hostname = Mail::MIMEDefang::Net::get_host_name();
-	  $body = "From: $DaemonName <$DaemonAddress>\n";
-	  $body .= "To: $Sender\n";
-	  $body .= Mail::MIMEDefang::RFC2822::gen_date_msgid_headers($QueueID, $hostname);
-	  $body .= "Auto-Submitted: auto-generated\n";
-	  $body .= "MIME-Version: 1.0\nContent-Type: text/plain\n";
-	  $body .= "Precedence: bulk\n";
-	  $body .= "Subject: $NotifySenderSubject\n\n";
-	  unless($NotifyNoPreamble) {
-	    $body .= "An e-mail you sent with message-id $MessageID\n";
-	    $body .= "was modified by our mail scanning software.\n\n";
-	    $body .= "The recipients were:";
-	    foreach my $recip (@Recipients) {
-		    $body .= " $recip";
-	    }
-	    $body .= "\n\n";
-	  }
-	  if (open(FILE, "<", "NOTIFICATION")) {
-	    unless($NotifyNoPreamble) {
-		    $body .= "Here are the details of the modification:\n\n";
-	    }
-	    while(<FILE>) {
-		    $body .= $_;
-	    }
-	    close(FILE);
-	  }
-	  send_mail($DaemonAddress, $DaemonName, $Sender, $body);
+    my $body = "";
+    unless($NotifyNoPreamble) {
+      $body .= "An e-mail you sent with message-id $MessageID\n";
+      $body .= "was modified by our mail scanning software.\n\n";
+      $body .= "The recipients were:";
+      foreach my $recip (@Recipients) {
+	    $body .= " $recip";
+      }
+      $body .= "\n\n";
+    }
+    my $notify;
+    if (open($notify, "<", "NOTIFICATION")) {
+      unless($NotifyNoPreamble) {
+	    $body .= "Here are the details of the modification:\n\n";
+      }
+      while(<$notify>) {
+	    $body .= $_;
+      }
+      close($notify);
+    }
+    my $mime = MIME::Entity->build(
+      From => "\"$DaemonName\" <$DaemonAddress>",
+      To => $Sender,
+      Subject => $NotifySenderSubject,
+      Encoding => 'quoted-printable',
+      Data => [$body],
+      Type => "text/plain");
+    $mime->head->add('Auto-Submitted', 'auto-generated');
+    $mime->head->add('Precedence', 'bulk');
+    send_mail($DaemonAddress, $DaemonName, $Sender, $mime->stringify);
   }
 
   # Send notification to administrator, if required
+  my $adm_notify;
   if (-r "ADMIN_NOTIFICATION") {
 	my $body = "";
-	  if (open(FILE, "<", "ADMIN_NOTIFICATION")) {
-	    $body .= join('', <FILE>);
-	    close(FILE);
+	  if (open($adm_notify, "<", "ADMIN_NOTIFICATION")) {
+	    $body .= join('', <$adm_notify>);
+	    close($adm_notify);
 	    send_admin_mail($NotifyAdministratorSubject, $body);
 	  }
   }
@@ -1081,15 +1131,16 @@ sub send_mail {
   my($pid);
 
   # Fork and exec for safety instead of involving shell
-  $pid = open(CHILD, "|-");
+  my $ch;
+  $pid = open($ch, "|-");
   if (!defined($pid)) {
 	  md_syslog('err', "Cannot fork to run sendmail");
 	  return;
   }
 
   if ($pid) {   # In the parent -- pipe mail message to the child
-	  print CHILD $body;
-	  close(CHILD);
+	  print $ch $body;
+	  close($ch);
 	  return;
   }
 
@@ -1097,7 +1148,7 @@ sub send_mail {
 
   # Direct stdout to stderr, or we will screw up communication with
   # the multiplexor..
-  open(STDOUT, ">&STDERR");
+  open(STDOUT, ">&", \*STDERR);
 
   my(@cmd);
   if ($fromAddr ne "") {
@@ -1125,6 +1176,60 @@ sub send_mail {
   # NOTREACHED
 }
 
+=item send_multipart_mail(fromAddr, fromName, recipient, subject, body_text, body_html, extra_headers)
+
+Sends a multipart mail message using Sendmail.
+
+Invokes Sendmail without involving the shell, so that shell metacharacters won't cause security problems.
+
+=cut
+
+#***********************************************************************
+# %PROCEDURE: send_multipart_mail
+# %ARGUMENTS:
+#  fromAddr -- address of sender
+#  fromName -- full name of sender
+#  recipient -- address of recipient
+#  subject -- email subject
+#  body_text -- text message newline-terminated
+#  body_html -- html message newline-terminated
+#  extra_headers -- optional extra headers to add to the email message
+# %RETURNS:
+#  Nothing
+# %DESCRIPTION:
+#  Sends a mail message using Sendmail.  Invokes Sendmail without involving
+#  the shell, so that shell metacharacters won't cause security problems.
+#***********************************************************************
+sub send_multipart_mail {
+  my ($fromAddr, $fromName, $recipient, $subject, $body_text, $body_html, $extra_headers) = @_;
+  my $body;
+
+  my @bset = ('A' .. 'Z');
+  my $boundary = join '' => map $bset[rand @bset], 1 .. 10;
+  if($fromAddr =~ /^\<(.*)\>$/) {
+    $fromAddr = $1;
+  }
+  if (defined $fromAddr and $fromAddr ne '') {
+    my $mime = MIME::Entity->build(
+      From => "\"$fromName\" <$fromAddr>",
+      To => $recipient,
+      Subject => $subject,
+      Type => "multipart/alternative");
+    $mime->attach(Data => $body_text,
+                  Type => "text/plain");
+    $mime->attach(Data => $body_html,
+                  Type => "text/html");
+    $mime->head->add('Auto-Submitted', 'auto-generated');
+    $mime->head->add('Precedence', 'bulk');
+    foreach (keys %{$extra_headers}) {
+      $mime->head->add($_, $extra_headers->{$_});
+    }
+    send_mail($fromAddr, $fromName, $recipient, $mime->stringify);
+  } else {
+    md_syslog("Warning", "send_mail_multipart called with empty \"fromAddr\" parameter");
+  }
+}
+
 =item send_admin_mail(subject, body)
 
 Sends a mail message to the administrator
@@ -1145,22 +1250,171 @@ sub send_admin_mail {
   my ($subject, $body) = @_;
 
   my $mail;
-  my $hostname = Mail::MIMEDefang::Net::get_host_name();
-  $mail = "From: $DaemonName <$DaemonAddress>\n";
-  $mail .= "To: \"$AdminName\" <$AdminAddress>\n";
-  $mail .= Mail::MIMEDefang::RFC2822::gen_date_msgid_headers($QueueID, $hostname);
-  $mail .= "Auto-Submitted: auto-generated\n";
-  $mail .= "MIME-Version: 1.0\nContent-Type: text/plain\n";
-  $mail .= "Precedence: bulk\n";
-  $mail .= "Subject: $subject\n\n";
-  $mail .= $body;
-
-  send_mail($DaemonAddress, $DaemonName, $AdminAddress, $mail);
+  my $mime = MIME::Entity->build(
+    To => "\"$AdminName\" <$AdminAddress>",
+    From => "$DaemonName <$DaemonAddress>",
+    Subject => $subject,
+    Encoding => 'quoted-printable',
+    Data => [$body],
+    Type => "text/plain");
+  $mime->head->add('Auto-Submitted', 'auto-generated');
+  $mime->head->add('Precedence', 'bulk');
+  send_mail($DaemonAddress, $DaemonName, $AdminAddress, $mime->stringify);
 }
 
-=back
+=item read_commands_file()
 
-=over 4
+This function should only be called from C<filter_sender> and
+C<filter_recipient>. This will read the C<COMMANDS> file (as
+described in L<mimedefang-protocol(7)>), and will fill or update the
+following global variables: $Sender, @Recipients, %RecipientMailers,
+$RelayAddr, $RealRelayAddr, $RelayHostname, $RealRelayHostname,
+$QueueID, $Helo, %SendmailMacros.
+
+If you do not call C<read_commands_file>, then the only information
+available in C<filter_sender> and C<filter_recipient> is that
+which is passed as an argument to the function.
+
+=cut
+
+#***********************************************************************
+# %PROCEDURE: read_commands_file
+# %ARGUMENTS:
+#  needf - if true, will return an error when no closing "F" was found.
+#          (optional, default is false). needf should not be set when
+#          called from within filter_relay, filter_sender, filter_recipient.
+# %RETURNS:
+#  true if parse went well,
+#  false otherwise
+# %DESCRIPTION:
+#  Parses the COMMANDS file, and sets these global variables based
+#  upon the contents of that file:
+#    $Sender
+#    @Recipients
+#    %RecipientMailers
+#    $SuspiciousCharsInHeaders
+#    $SuspiciousCharsInBody
+#    $RelayAddr
+#    $RealRelayAddr
+#    $WasResent
+#    $RelayHostname
+#    $RealRelayHostname
+#    $QueueID
+#    $Subject
+#    $MessageID
+#    $Helo
+#    %SendmailMacros
+#
+#***********************************************************************
+sub read_commands_file {
+    my $needF = shift;
+    $needF = 0 unless defined($needF);
+
+    my $in;
+    if (!open($in, "<", "COMMANDS")) {
+	    fatal("Cannot open COMMANDS file from mimedefang: $!");
+	    return 0;
+    }
+
+    my($cmd, $arg, $rawcmd, $rawarg, $seenF);
+
+    # Save current recipient if called from filter_recipient
+    my @tmp_recipients = @Recipients;
+    @Recipients = ();
+    $seenF = 0;
+    my $recent_recip = "";
+
+    while(<$in>) {
+	    chomp;
+	    $rawcmd = $_;
+	    $cmd = percent_decode($rawcmd);
+	    $arg = substr($cmd, 1);
+	    $cmd = substr($cmd, 0, 1);
+	    $rawarg = substr($rawcmd, 1);
+
+	    if ($cmd eq "S") {
+	      $Sender = $arg;
+	    } elsif ($cmd eq "s") {
+	      push(@SenderESMTPArgs, $arg);
+	    } elsif ($cmd eq "F") {
+	      $seenF = 1;
+	      last;
+	    } elsif ($cmd eq "R") {
+	      my($recip, $rcpt_mailer, $rcpt_host, $rcpt_addr);
+	      ($recip, $rcpt_mailer, $rcpt_host, $rcpt_addr) = split(' ', $rawarg);
+	      $rcpt_mailer = "?" unless (defined($rcpt_mailer) and ($rcpt_mailer ne ""));
+	      $rcpt_host = "?" unless (defined($rcpt_host) and ($rcpt_host ne ""));
+	      $rcpt_addr = "?" unless (defined($rcpt_addr) and ($rcpt_addr ne ""));
+	      $recip = percent_decode($recip);
+	      $rcpt_mailer = percent_decode($rcpt_mailer);
+	      $rcpt_host = percent_decode($rcpt_host);
+	      $rcpt_addr = percent_decode($rcpt_addr);
+	      push(@Recipients, $recip);
+	      $RecipientMailers{$recip} = [$rcpt_mailer, $rcpt_host, $rcpt_addr];
+	      $recent_recip = $recip;
+	    } elsif ($cmd eq "r") {
+	      push (@{$RecipientESMTPArgs{$recent_recip}}, $arg);
+	    } elsif ($cmd eq "!") {
+	      $SuspiciousCharsInHeaders = 1;
+	    } elsif ($cmd eq "?") {
+	      $SuspiciousCharsInBody    = 1;
+	    } elsif ($cmd eq "I") {
+	      $RelayAddr = $arg;
+	      $RealRelayAddr = $arg;
+	    } elsif ($cmd eq "J") {
+	      $WasResent = 1;
+	      $RelayAddr = $arg;
+	      my($iaddr, $iname);
+	      $iaddr = inet_aton($RelayAddr);
+	      $iname = gethostbyaddr($iaddr, AF_INET);
+	      if (defined($iname)) {
+		      $RelayHostname = $iname;
+	      } else {
+		      $RelayHostname = "[$RelayAddr]";
+	      }
+	    } elsif ($cmd eq "H") {
+	      $RelayHostname = $arg;
+	      $RealRelayHostname = $arg;
+	    } elsif ($cmd eq "Q") {
+	      $QueueID = $arg;
+	    } elsif ($cmd eq "U") {
+	      $SubjectCount++;
+	      if ($SubjectCount > 1) {
+		      md_syslog('warning', "Message contains more than one Subject: header: $Subject --> $arg");
+  	    } else {
+		      $Subject = $arg;
+	      }
+	    } elsif ($cmd eq "X") {
+	      $MessageID = $arg;
+	    } elsif ($cmd eq "E") {
+	      $Helo = $arg;
+	    } elsif ($cmd eq "=") {
+	      my($macro, $value);
+	      ($macro, $value) = split(' ', $rawarg);
+	      $value = "" unless defined($value);
+	      $macro = "" unless defined($macro);
+	      if ($macro ne "") {
+		      $macro = percent_decode($macro);
+		      $value = percent_decode($value);
+		      $SendmailMacros{$macro} = $value;
+	      }
+	    } elsif ($cmd eq "i") {
+		    $MIMEDefangID = $arg;
+	    } else {
+	      md_syslog('warning', "Unknown command $cmd from mimedefang");
+	    }
+    }
+    close($in);
+
+    if ( $needF && !$seenF ) {
+	    md_syslog('err', "COMMANDS file from mimedefang did not terminate with 'F' -- check disk space in spool directory");
+	    fatal("COMMANDS file did not end with F");
+      return 0;
+    }
+
+    push @Recipients, @tmp_recipients;
+    return 1;
+}
 
 =back
 
@@ -1176,7 +1430,7 @@ L<Mail::MIMEDefang::DKIM(3)>
 
 L<Mail::MIMEDefang::DKIM::ARC(3)>
 
-L<Mail::MIMEDefang::DKIM::Authres(3)>
+L<Mail::MIMEDefang::Authres(3)>
 
 L<Mail::MIMEDefang::Mail(3)>
 
